/*
 * Copyright (c) 1999, 2017, Oracle and/or its affiliates. All rights reserved.
 */

package com.oracle.jmc.flightrecorder.pde.simplerule;

import java.net.URL;
import java.util.Locale;
import java.util.ResourceBundle;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.pde.core.plugin.IMatchRules;
import org.eclipse.pde.core.plugin.IPluginBase;
import org.eclipse.pde.core.plugin.IPluginElement;
import org.eclipse.pde.core.plugin.IPluginExtension;
import org.eclipse.pde.core.plugin.IPluginModelBase;
import org.eclipse.pde.core.plugin.IPluginModelFactory;
import org.eclipse.pde.core.plugin.IPluginReference;
import org.eclipse.pde.ui.IFieldData;
import org.eclipse.pde.ui.templates.OptionTemplateSection;
import org.osgi.framework.Bundle;

import com.oracle.jmc.flightrecorder.pde.Activator;
import com.oracle.jmc.pde.PluginReference;

public class SimpleRuleTemplateSection extends OptionTemplateSection {
	private static final String EXTENSION_POINT_ID_RULE_PROVIDER = "com.oracle.jmc.flightrecorder.rules.ruleProvider"; //$NON-NLS-1$
	public static final String KEY_CLASS_NAME = "className"; //$NON-NLS-1$
	public static final String DEFAULT_CLASS_NAME = "SimpleJfrRule"; //$NON-NLS-1$
	public static final String KEY_RULE_NAME = "ruleName"; //$NON-NLS-1$
	public static final String DEFAULT_RULE_NAME = "Simple JFR Rule"; //$NON-NLS-1$

	public SimpleRuleTemplateSection() {
		setPageCount(1);
		createOptions();
	}

	@Override
	public String getSectionId() {
		return "simpleruletemplate"; //$NON-NLS-1$
	}

	/*
	 * @see ITemplateSection#getNumberOfWorkUnits()
	 */
	@Override
	public int getNumberOfWorkUnits() {
		return super.getNumberOfWorkUnits() + 1;
	}

	private void createOptions() {
		addOption(KEY_PACKAGE_NAME, Messages.SimpleRuleTemplateSection_PACKAGE_LABEL, (String) null, 0);
		addOption(KEY_CLASS_NAME, Messages.SimpleRuleTemplateSection_CLASS_NAME_LABEL, DEFAULT_CLASS_NAME, 0);
		addOption(KEY_RULE_NAME, Messages.SimpleRuleTemplateSection_RULE_NAME_LABEL, DEFAULT_RULE_NAME, 0);
	}

	@Override
	public void addPages(Wizard wizard) {
		WizardPage page = createPage(0, "formpageContextID"); //$NON-NLS-1$
		page.setTitle(Messages.SimpleRuleTemplateSection_PAGE_TITLE);
		page.setDescription(Messages.SimpleRuleTemplateSection_PAGE_DESCRIPTION);
		wizard.addPage(page);
		markPagesAdded();
	}

	@Override
	public boolean isDependentOnParentWizard() {
		return true;
	}

	@Override
	protected void initializeFields(IFieldData data) {
		// In a new project wizard, we don't know this yet - the model has not been created
		String id = data.getId();
		initializeOption(KEY_PACKAGE_NAME, getFormattedPackageName(id));
	}

	@Override
	public void initializeFields(IPluginModelBase model) {
		// In the new extension wizard, the model exists so we can initialize directly from it
		String pluginId = model.getPluginBase().getId();
		initializeOption(KEY_PACKAGE_NAME, getFormattedPackageName(pluginId));
	}

	@Override
	public String getUsedExtensionPoint() {
		return EXTENSION_POINT_ID_RULE_PROVIDER;
	}

	@Override
	protected void updateModel(IProgressMonitor monitor) throws CoreException {
		IPluginBase plugin = model.getPluginBase();
		IPluginExtension extension = createExtension(EXTENSION_POINT_ID_RULE_PROVIDER, true);
		IPluginModelFactory factory = model.getPluginFactory();

		IPluginElement setElement = factory.createElement(extension);
		setElement.setName("ruleProvider"); //$NON-NLS-1$
		extension.add(setElement);
		if (!extension.isInTheModel()) {
			plugin.add(extension);
		}
	}

	protected String getFormattedPackageName(String id) {
		StringBuffer buffer = new StringBuffer();
		for (int i = 0; i < id.length(); i++) {
			char ch = id.charAt(i);
			if (buffer.length() == 0) {
				if (Character.isJavaIdentifierStart(ch)) {
					buffer.append(ch);
				}
			} else {
				if (Character.isJavaIdentifierPart(ch) || ch == '.') {
					buffer.append(ch);
				}
			}
		}
		String packageName = buffer.toString().toLowerCase(Locale.ENGLISH);

		if (packageName.length() != 0) {
			return packageName + ".rules"; //$NON-NLS-1$
		}
		return "rules"; //$NON-NLS-1$

	}

	@Override
	public IPluginReference[] getDependencies(String schemaVersion) {
		// Ensure schema version was defined
		if (schemaVersion == null) {
			return super.getDependencies(null);
		}
		// Create the dependencies
		IPluginReference[] dependencies = new IPluginReference[2];
		dependencies[0] = new PluginReference("com.oracle.jmc.flightrecorder.rules", "6.0", IMatchRules.GREATER_OR_EQUAL); //$NON-NLS-1$ //$NON-NLS-2$
		dependencies[1] = new PluginReference("com.oracle.jmc.flightrecorder", "6.0", IMatchRules.GREATER_OR_EQUAL); //$NON-NLS-1$ //$NON-NLS-2$
		return dependencies;
	}

	@Override
	protected ResourceBundle getPluginResourceBundle() {
		Bundle bundle = Platform.getBundle(com.oracle.jmc.flightrecorder.pde.Activator.PLUGIN_ID);
		return Platform.getResourceBundle(bundle);
	}

	@Override
	protected URL getInstallURL() {
		return Activator.getDefault().getInstallURL();
	}

	@Override
	public String[] getNewFiles() {
		return new String[0];
	}

}
